@@ -24,6 +24,7 @@ from tinycoder.git_manager import GitManager
 from tinycoder.file_manager import FileManager
 from tinycoder.llms.base import LLMClient
 from tinycoder.llms import create_llm_client
+from tinycoder.llms.pricing import get_model_pricing
 from tinycoder.prompt_builder import PromptBuilder
 from tinycoder.repo_map import RepoMap
 from tinycoder.rule_manager import RuleManager
@@ -232,19 +233,21 @@ class App:
         self.console_interface = ConsoleInterface(
             logger=self.logger,
             get_app_mode_func=lambda: self.mode, # Pass a callable to get current mode
-            readline_available=READLINE_AVAILABLE # Pass the app-level constant
+            readline_available=READLINE_AVAILABLE, # Pass the app-level constant
+            get_token_breakdown_func=self._get_current_context_token_breakdown # Pass token breakdown function
         )
         self.logger.debug("ConsoleInterface initialized.")
 
     def _init_app_state(self) -> None:
         """Initializes basic application state variables."""
         self.coder_commits: Set[str] = set()
-        self.coder_commits: Set[str] = set()
         self.mode = "code" # Default mode
         self.lint_errors_found: Dict[str, str] = {}
         self.reflected_message: Optional[str] = None
         self.include_repo_map: bool = True # Default to including the repo map
-        self.logger.debug("Basic app state initialized (commits, mode, lint status, repo map toggle).")
+        self.total_input_tokens: int = 0
+        self.total_output_tokens: int = 0
+        self.logger.debug("Basic app state initialized (commits, mode, lint status, repo map toggle, usage tracking).")
 
     def toggle_repo_map(self, state: bool) -> None:
         """Sets the state for including the repo map in prompts."""
@@ -342,6 +345,7 @@ class App:
             file_manager=self.file_manager,
             git_manager=self.git_manager,
             docker_manager=self.docker_manager,
+            logger=self.logger,
             clear_history_func=self.history_manager.clear,
             write_history_func=self.history_manager.save_message_to_file_only,
             get_mode=lambda: self.mode,
@@ -633,6 +637,50 @@ class App:
         except Exception as e:
             self.logger.error(f"Failed to save readline history to {self.history_file}: {e}")
 
+    def _get_current_context_token_breakdown(self) -> Dict[str, int]:
+        """Calculates the approximate token count breakdown for the current context."""
+        # This is a helper function to estimate tokens from characters.
+        def count_tokens(text: str) -> int:
+            return int(len(text) / 4)
+
+        # 1. System Prompt (Base + Rules)
+        active_rules = self.rule_manager.get_active_rules_content()
+        # Build system prompt WITHOUT repo map to isolate its size
+        base_system_prompt_content = self.prompt_builder.build_system_prompt(
+            self.mode,
+            active_rules,
+            include_map=False # Exclude map for this part of calculation
+        )
+        system_prompt_tokens = count_tokens(base_system_prompt_content)
+
+        # 2. Repo Map
+        repo_map_tokens = 0
+        if self.include_repo_map:
+            # Generate the repo map separately to get its size
+            chat_files_rel = self.file_manager.get_files()
+            repo_map_str = self.repo_map.generate_map(chat_files_rel)
+            repo_map_tokens = count_tokens(repo_map_str)
+
+        # 3. File Context
+        file_context_message = self.prompt_builder.get_file_content_message()
+        file_context_content = file_context_message['content'] if file_context_message else ""
+        file_context_tokens = count_tokens(file_context_content)
+
+        # 4. History
+        current_history = self.history_manager.get_history()
+        history_content = "\n".join(msg['content'] for msg in current_history)
+        history_tokens = count_tokens(history_content)
+
+        total_tokens = system_prompt_tokens + repo_map_tokens + file_context_tokens + history_tokens
+
+        return {
+            "total": total_tokens,
+            "prompt_rules": system_prompt_tokens,
+            "repo_map": repo_map_tokens,
+            "files": file_context_tokens,
+            "history": history_tokens,
+        }
+
     def _send_to_llm(self) -> Optional[str]:
         """Sends the current chat history and file context to the LLM."""
         current_history = self.history_manager.get_history()
@@ -701,9 +749,11 @@ class App:
                     "System prompt not found at the beginning of messages for LLM."
                 )
 
-            total_tokens = (sum(len(msg["content"]) for msg in history_to_send) + len(system_prompt_text))/4
+            input_chars = sum(len(msg["content"]) for msg in history_to_send) + len(system_prompt_text)
+            input_tokens = round(input_chars / 4)
+            self.total_input_tokens += input_tokens
             
-            self.logger.debug(f"Approx. total tokens to send: {total_tokens}") # Changed to debug for less noise
+            self.logger.debug(f"Approx. input tokens to send: {input_tokens}")
 
             self.spinner.start()
             try:
@@ -733,8 +783,10 @@ class App:
                 else:
                     self.logger.info(f"\n{FmtColors['CYAN']}{STYLES['BOLD']}ASSISTANT{RESET}:\n" + response_content + "\n")
                 
-                n_tokens = len(response_content)/4 # Based on raw response
-                self.logger.debug("Approx. response tokens: %d", n_tokens) # Changed to debug
+                output_chars = len(response_content)
+                output_tokens = round(output_chars / 4) # Based on raw response
+                self.total_output_tokens += output_tokens
+                self.logger.debug("Approx. response tokens: %d", output_tokens)
             
                 return response_content
 
@@ -936,6 +988,66 @@ class App:
 
         return False
 
+    def _display_usage_summary(self):
+        """Calculates and displays the token usage and estimated cost for the session."""
+        if self.total_input_tokens == 0 and self.total_output_tokens == 0:
+            return  # Don't display anything if no API calls were made
+
+        pricing = get_model_pricing(self.model)
+        total_tokens = self.total_input_tokens + self.total_output_tokens
+        
+        cost_line = ""
+        if pricing:
+            input_cost = (self.total_input_tokens / 1_000_000) * pricing["input"]
+            output_cost = (self.total_output_tokens / 1_000_000) * pricing["output"]
+            total_cost = input_cost + output_cost
+            cost_line = f"Est. Cost:  {STYLES['BOLD']}{FmtColors['YELLOW']}${total_cost:.4f}{RESET}"
+        elif self.model:
+            cost_line = f"Est. Cost:  {FmtColors['GREY']}(price data unavailable for {self.model}){RESET}"
+            
+        # Prepare formatted lines
+        title_line = f"{STYLES['BOLD']}Session Summary{RESET}"
+        model_line = f"Model:      {STYLES['BOLD']}{FmtColors['GREEN']}{self.model}{RESET}"
+        tokens_line = (
+            f"Tokens:     {STYLES['BOLD']}{FmtColors['CYAN']}{total_tokens:,}{RESET} "
+            f"{FmtColors['GREY']}(Input: {self.total_input_tokens:,} | Output: {self.total_output_tokens:,}){RESET}"
+        )
+
+        def get_visual_length(s: str) -> int:
+            """Calculates the visible length of a string by removing ANSI escape codes."""
+            return len(re.sub(r'\x1b\[[0-9;]*m', '', s))
+
+        # Build the final output string with a box
+        border_char = '─'
+        width = 60 # Fixed width for simplicity and clean alignment
+
+        def pad_line(line: str) -> str:
+            """Pads a line to the fixed width, accounting for ANSI codes."""
+            visual_len = get_visual_length(line)
+            padding = ' ' * (width - visual_len)
+            return f"{line}{padding}"
+        
+        def center_line(line: str) -> str:
+            """Centers a line within the fixed width, accounting for ANSI codes."""
+            visual_len = get_visual_length(line)
+            padding_total = width - visual_len
+            if padding_total < 0: padding_total = 0
+            left_pad = ' ' * (padding_total // 2)
+            right_pad = ' ' * (padding_total - len(left_pad))
+            return f"{left_pad}{line}{right_pad}"
+
+        summary_message = (
+            f"\n{FmtColors['GREY']}┌{border_char * (width + 2)}┐{RESET}\n"
+            f"{FmtColors['GREY']}│ {center_line(title_line)} {FmtColors['GREY']}│{RESET}\n"
+            f"{FmtColors['GREY']}├{border_char * (width + 2)}┤{RESET}\n"
+            f"{FmtColors['GREY']}│ {pad_line(model_line)} {FmtColors['GREY']}│{RESET}\n"
+            f"{FmtColors['GREY']}│ {pad_line(tokens_line)} {FmtColors['GREY']}│{RESET}\n"
+            f"{FmtColors['GREY']}│ {pad_line(cost_line)} {FmtColors['GREY']}│{RESET}\n"
+            f"{FmtColors['GREY']}└{border_char * (width + 2)}┘{RESET}"
+        )
+        # Use print() to ensure the summary is always visible regardless of log level
+        print(summary_message)
+
     def process_user_input(self, non_interactive: bool = False):
         """Processes the latest user input (already in history), sends to LLM, handles response."""
         response = self._send_to_llm()
@@ -1279,7 +1391,6 @@ class App:
         
         # Constructing the welcome message parts
         model_line = f"  Model: {FmtColors['GREEN']}{STYLES['BOLD']}{self.model}{RESET}"
-        repo_map_line = f"  Repo Map: {repo_map_status_output_str}"
 
         # Active Rules status
         num_active_rules = len(self.rule_manager.last_loaded_rule_names)
@@ -1319,7 +1430,6 @@ class App:
 
         # Log each part on a new line for clarity
         self.logger.info(model_line)
-        self.logger.info(repo_map_line)
         self.logger.info(docker_line) # Added Docker status line
         self.logger.info(active_rules_line)
         self.logger.info(help_line)
@@ -1371,4 +1481,5 @@ class App:
                 print("\nExiting (EOF).", file=sys.stderr)
                 break  # Exit on Ctrl+D
 
+        self._display_usage_summary()
         self.logger.info("Goodbye! 👋")
